import 'package:flutter/widgets.dart';
import 'package:widget/src/constant.dart';

class BattleForeground extends StatelessWidget {
  const BattleForeground({super.key});

  @override
  Widget build(BuildContext context) {
    return CustomPaint(
      painter: _BattleForegroundPainter(),
      child: const SizedBox.expand(),
    );
  }
}

class _BattleForegroundPainter extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()
      ..color = kBoardForegroundColor
      ..style = PaintingStyle.fill;

    paint.color = kBoardForegroundColor;
    canvas.drawPath(_getBoardPath(size), paint);
  }

  Path _getBoardPath(Size size) {
    Path resultPath = Path();
    final boardRRect = RRect.fromRectAndRadius(Offset.zero & size, const Radius.circular(kBoardRadius));

    resultPath.addRRect(boardRRect);
    resultPath = Path.combine(PathOperation.difference, resultPath, _getCardPath(size));
    resultPath = Path.combine(PathOperation.difference, resultPath, _getBottomButtonPath(size));
    resultPath = Path.combine(PathOperation.difference, resultPath, _getRightButtonPath(size));
    return resultPath;
  }

  // 除去卡片的部分
  Path _getCardPath(Size size) {
    Path path = Path();
    final cardRect = Rect.fromLTWH(
      size.width * (kLeftBorderAspectRatio),
      0,
      size.width * (kCardWidthAspectRatio - 2 * kCardCoverAspectRatio),
      size.height * (kCardHeightAspectRatio - kCardOverflowAspectRatio),
    );

    path.addRRect(RRect.fromRectAndCorners(
      cardRect,
      bottomLeft: const Radius.circular(kCardRadius),
      bottomRight: const Radius.circular(kCardRadius),
    ));

    return path;
  }

  // 除去底部按钮部分的部分
  Path _getBottomButtonPath(Size size) {
    final double left = size.width * kBottomButtonLeftAspectRatio;
    final double right = size.width * kBottomButtonRightAspectRatio;
    final double top = size.height * kBottomButtonTopAspectRatio;
    final double bottom = size.height * kBottomButtonBottomAspectRatio;
    final double line = size.width * kButtonLineAspectRatio;

    final double rightSide = size.width * kRightButtonRightAspectRatio;

    final double spacing = ((right - left) - line * 6) / 5 + line;
    final path = Path();
    final offset = Offset(left, top);

    // 底部按钮的横线部分
    path.addRRect(RRect.fromLTRBR(left, top, rightSide, top + line, const Radius.circular(100)));

    final double lineHeight = bottom - top - line;
    final Size lineSize = Size(line, lineHeight);
    path.addRRect(RRect.fromRectAndRadius((offset + const Offset(0, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(spacing, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(spacing * 2, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(spacing * 3, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(spacing * 4, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(spacing * 5, 0)) & lineSize, const Radius.circular(100)));
    return path;
  }

  Path _getRightButtonPath(Size size) {
    final double left = size.width * kRightButtonLeftAspectRatio;
    final double right = size.width * kRightButtonRightAspectRatio;
    final double top = size.height * kRightButtonTopAspectRatio;
    final double bottom = size.height * kRightButtonBottomAspectRatio;
    final double line = size.width * kButtonLineAspectRatio;

    final double bottomSide = size.height * kBottomButtonTopAspectRatio + line;

    final double spacing = ((bottom - top) - line * 6) / 5 + line;
    final path = Path();
    final offset = Offset(left, top);

    // 右侧按钮的横线部分
    path.addRRect(RRect.fromLTRBR(right - line, top, right, bottomSide, const Radius.circular(100)));

    final double lineWidth = right - left;
    final Size lineSize = Size(lineWidth, line);
    path.addRRect(RRect.fromRectAndRadius((offset + const Offset(0, 0)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(0, spacing)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(0, spacing * 2)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(0, spacing * 3)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(0, spacing * 4)) & lineSize, const Radius.circular(100)));
    path.addRRect(RRect.fromRectAndRadius((offset + Offset(0, spacing * 5)) & lineSize, const Radius.circular(100)));
    return path;
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) => false;
}
